/*
 * ascii2array 13.10.2006 sm
 *
 * to compile use the mex script
 *  mex -setup  (first time)
 *  mex ascii2array.c  (will build ascii2array.mexw32)
 *
 * be careful: c/i386 use different precision on double variables
 * don't use stupid comparisons
 * (matlab:) 5e10 != (c:) 5e10
 *
*/

/* $Revision: 0.01 $ */

#include <math.h>
#include "mex.h" 

/* If you are using a compiler that equates NaN to zero, you must
 * compile this example using the flag -DNAN_EQUALS_ZERO. For 
 * example:
 *
 *     mex -DNAN_EQUALS_ZERO findnz.c  
 *
 * This will correctly define the IsNonZero macro for your 
   compiler. */

#if NAN_EQUALS_ZERO
#define IsNonZero(d) ((d) != 0.0 || mxIsNaN(d))
#else
#define IsNonZero(d) ((d) != 0.0)
#endif

enum aaState { errorS=0,spaceS,negS,mainS,subS,expnegS,expS };

  /* Declare variables. */
  int elements, i, dim_x, add_one_dim, dim_x_const, dim_y, linefeed, state, expsign, exponent, exponent2, index;
  double  mantisse, res, sign;
  unsigned int intc,lineM,lineN;
  char *pinput;
  double *pind;

void store() {
    index = (dim_y*lineN)+lineM;
    mantisse*= sign;
    res=pow(10,exponent+(expsign*exponent2));
    res*= mantisse;
    /*mexPrintf("res: %20.10f  %20.10e\n",res,res);*/
    pind[index]= res;
    lineN++;
    if (lineN >= dim_x_const) {
        lineN=0;
        lineM++;
    }
    sign=1; expsign=1; mantisse=0; exponent=0; exponent2=0;
    state=spaceS;
}

void mexFunction(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
	

  /* Check for proper number of input and output arguments. */    
  if (nrhs != 1) {
    mexErrMsgTxt("One input argument required.");
  } 
  if (nlhs > 1) {
    mexErrMsgTxt("Too many output arguments.");
  }

  /* Check data type of input argument. */
  if (!(mxIsUint8(prhs[0]))) {
    mexErrMsgTxt("Input array must be of type uint8.");
  }
    
  /* Get the number of elements in the input argument. */
  elements = mxGetNumberOfElements(prhs[0]);

  /* Get the data. */
  pinput = (char *)mxGetChars(prhs[0]);
  
  i=0; dim_x=0; add_one_dim=0; dim_x_const=0; dim_y=0; linefeed=0;
  
  while(i<elements) {
      /*mexPrintf("%c",(char)pinput[i]);*/
      switch(pinput[i]) {
          case('-'):
          case('+'):
          case('.'):
          case('e'):
          case('E'):
          case('0'):
          case('1'):
          case('2'):
          case('3'):
          case('4'):
          case('5'):
          case('6'):
          case('7'):
          case('8'):
          case('9'):
              linefeed=0;
              add_one_dim = 1;
              break;
          case(0x0d):
          case(0x0a):
              if (linefeed==1) {
                  linefeed=0;
                  break;                  
              }               
              linefeed=1;
              
              if (add_one_dim == 1) {
                  dim_x++;
                  add_one_dim = 0;
              }
              /* calculate dim_x_const once */
              if (dim_x_const == 0) {
                  dim_x_const = dim_x;
              } else
              if (dim_x_const != dim_x) {
                 mexPrintf("Array size: x: %i (now: %i), line: %i\n",dim_x_const,dim_x,dim_y);                 
                 mexErrMsgTxt("Pass1: Input array trash - no constant array size.");
                 break;
              }
              dim_x = 0;
              dim_y++;
              break;
          case(0x09):              
          case(' '):
              if (add_one_dim == 1) {
                  dim_x++;
                  add_one_dim = 0;
              }
              linefeed=0;
              break;
         default:
             mexPrintf("Error in line %i\n",dim_y);
             mexErrMsgTxt("Pass1: Input array trash.");
             break;
      }
      i++;
  }
/*  mexPrintf("Array size: x: %i, y: %i\n",dim_x_const,dim_y);*/
  
  plhs[0] = mxCreateDoubleMatrix(dim_y, dim_x_const, mxREAL);
  pind = mxGetPr(plhs[0]);

    state=spaceS;
    lineN=0; lineM=0;
    sign=1; expsign=1; mantisse=0; exponent=0; exponent2=0;
    i=0;
  
  while(i<elements) {
/*      mexPrintf("state: %i, char: %i(%c), res: %i %e e %i+(%i*%i) \n",
        state, (char)pinput[i], (char)pinput[i], sign, mantisse, exponent, expsign, exponent2);
*/
      switch(state) {
        case errorS: /* error*/
            mexPrintf("input error (char: [%c]) in line %i\n", pinput[i-1], lineM+1);
            mexErrMsgTxt("Pass2: input error - terminating\n");
            return;
            break; /*case errorS*/

        case spaceS: /* white space */
            switch(pinput[i]) {
                case('0'):
                case('1'):
                case('2'):
                case('3'):
                case('4'):
                case('5'):
                case('6'):
                case('7'):
                case('8'):
                case('9'):
                    mantisse+=pinput[i]-'0';
                    state=mainS;
                    break;
                case('+'):
                    state=mainS;
                    break;
                case('-'):
                    sign *= -1;
                    state=negS;
                    break;
                case(0x0d):
                case(0x0a):
                case(0x09):
                case(' '):
                    /*ignore, keep state*/
                    break;
                case('.'):
                    state=subS;
                    break;
                case('e'):
                case('E'):
                default:
                    state=errorS;
                    break;
            } /*switch */
            break; /*case*/

        case negS: /* sign */
            switch(pinput[i]) {
                case('0'):
                case('1'):
                case('2'):
                case('3'):
                case('4'):
                case('5'):
                case('6'):
                case('7'):
                case('8'):
                case('9'):
                    mantisse+=pinput[i]-'0';
                    state=mainS;
                    break;
                case('-'):
                    sign *= -1;
                    /*keep state */
                    break;
                case('.'):
                    state=subS;
                    break;
                case(' '):
                case(0x09):
                case('+'):
                case(0x0d):
                case(0x0a):
                case('e'):
                case('E'):
                default:
                    state=errorS;
                    break;
            } /* switch */
            break; /*case negS */

        case mainS: /* xx. mantisse */
            switch(pinput[i]) {
                case('0'):
                case('1'):
                case('2'):
                case('3'):
                case('4'):
                case('5'):
                case('6'):
                case('7'):
                case('8'):
                case('9'):
                    mantisse*=10;
                    mantisse+=pinput[i]-'0';
                    /*keep state */
                    break;
                case('e'):
                case('E'):
                    state=expnegS;
                    break;
                case(0x0d):
                case(0x0a):
                case(0x09):
                case(' '):
										store();
                    break;
                case('.'):
                    state=subS;
                    break;
                case('-'):
                case('+'):
                default:
                    state=errorS;
                    break;
            } /* switch*/
            break; /*case mainS*/

        case subS: /* .xx mantisse*/
            switch(pinput[i]) {
                case('0'):
                case('1'):
                case('2'):
                case('3'):
                case('4'):
                case('5'):
                case('6'):
                case('7'):
                case('8'):
                case('9'):
                    mantisse*=10;
                    mantisse+=pinput[i]-'0';
                    exponent--;
                    /*keep state*/
                    break;
                case('e'):
                case('E'):
                    state=expnegS;
                    break;
                case(0x0d):
                case(0x0a):
                case(0x09):
                case(' '):
                		store();
                    break;
                case('+'):
                case('.'):
                case('-'):
                default:
                    state=errorS;
                    break;
            } /* switch*/
            break; /*case subS*/

        case expnegS: /* sign exponent*/
            switch(pinput[i]) {
                case('0'):
                case('1'):
                case('2'):
                case('3'):
                case('4'):
                case('5'):
                case('6'):
                case('7'):
                case('8'):
                case('9'):
                    exponent2*=10;
                    exponent2+= pinput[i]-'0';
                    break;
                case('+'):
                    state=expS;
                    break;
                case('-'):
                    expsign *= -1;
                    /*keep state*/
                    break;
                case(0x0d):
                case(0x0a):
                case(0x09):
                case(' '):
                		store();
                    break;
                case('.'):
                case('e'):
                case('E'):
                default:
                    state=errorS;
                    break;
            } /* switch*/
            break; /*case expnegS*/

        case expS: /* exponent*/
            switch(pinput[i]) {
                case('0'):
                case('1'):
                case('2'):
                case('3'):
                case('4'):
                case('5'):
                case('6'):
                case('7'):
                case('8'):
                case('9'):
                    exponent2*=10;
                    exponent2+=pinput[i]-'0';
                    /*keep state*/
                    break;
                case(0x0d):
                case(0x0a):
                case(0x09):
                case(' '):
                		store();
                    break;
                case('.'):
                case('e'):
                case('E'):
                case('+'):
                case('-'):
                default:
                    state=errorS;
                    break;
            } /* switch*/
            break; /*case expS*/

        default:
            mexPrintf("state machine error(%i) in line %i\n", state, lineM);
            mexErrMsgTxt("Pass2: sorry this should not happen!");
            return;
            break; /* default*/
        } /* switch(state)*/
        i++;
    }/* while(i<elements)*/
  
} /* main*/


