function [ data ] = psv_edit_optics( data )
    if nargin ~= 1; return; end;

%
% if structure ps does carry beam data return 
if(isfield(data.ps(data.my_axes),'b_stat'))
else
    uiwait(msgbox('No phase space data!','PSV_SUM_PLOT','error'))
    return
end
%
% unpack
phasespace  = data.ps(data.my_axes).phasespace;
x  = data.ps(data.my_axes).b_stat.x;
b_twiss = data.ps(data.my_axes).b_stat.b_twiss;
gammag = data.ps(data.my_axes).gamma;
%
prompt = {
    'Horizontal /alpha: ';...
    'Horizontal /beta: ';...
    'Vertical /alpha: ';...
    'Vertical /beta: ';...
    };
defaultanswer = {
    num2str(b_twiss(1,1));... = 0;
    num2str(b_twiss(2,1));... = 3;
    num2str(b_twiss(1,3));...
    num2str(b_twiss(2,3))...
    };
name = 'Edit Twiss Parameters';
numlines = 1;
answer = inputdlg(prompt,name,numlines,defaultanswer);
if(isempty(answer))
    return
end
%
b_twiss_new(1,1) = str2num(char(answer(1)));
b_twiss_new(2,1) = str2num(char(answer(2)));
b_twiss_new(1,3) = str2num(char(answer(3)));
b_twiss_new(2,3) = str2num(char(answer(4)));
%
M=diag(ones(1,6));
%M_trans = [sqrt(beta_new/beta_old) 0. ; ...
%    -(alpha_new - alpha_old)/sqrt(beta_new*beta_old) sqrt(beta_old/beta_new)];

M_trans_h = [sqrt(b_twiss_new(2,1)/b_twiss(2,1)) 0. ; ...
    -(b_twiss_new(1,1) - b_twiss(1,1))/sqrt(b_twiss_new(2,1)*b_twiss(2,1)) sqrt(b_twiss(2,1)/b_twiss_new(2,1))];
M_trans_v = [sqrt(b_twiss_new(2,3)/b_twiss(2,3)) 0. ; ...
    -(b_twiss_new(1,3) - b_twiss(1,3))/sqrt(b_twiss_new(2,3)*b_twiss(2,3)) sqrt(b_twiss(2,3)/b_twiss_new(2,3))];
%
M(1:2,1:2) = M_trans_h;
M(3:4,3:4) = M_trans_v;

buffer = zeros(length(x),6);

for i=1:length(x)
    buffer(i,:)=(M*x(i,:)')';
end
x = buffer;
%
% put into phasespace
for i=1:6
   phasespace(:,i) = x(:,i) + data.ps(data.my_axes).b_stat.b_mean(i);
end
%
% re-calculate phase space parameters
[x,b_mean,b_rms,b_emit,b_twiss,weight] = beam_statistics(phasespace,gammag);
%
% put into structure 'ps'
data.ps(data.my_axes).b_stat.x = x;
data.ps(data.my_axes).b_stat.b_mean = b_mean;
data.ps(data.my_axes).b_stat.b_rms = b_rms;
data.ps(data.my_axes).b_stat.b_emit = b_emit;
data.ps(data.my_axes).b_stat.b_twiss = b_twiss;
data.ps(data.my_axes).b_stat.weight = weight;
%--- phasespace
data.ps(data.my_axes).phasespace = phasespace;
%
%--- plot
data = psv_plot_ps(data, 0);
