%--------------------------------------------------------------------------
%-------    load phase space ----------------------------------------------
% is called after opening files, from sliders, from psv_all_mesh_plot and 
% from psv_show_movie   
%--------------------------------------------------------------------------
%
function [ data ] = psv_get_ps(data, usingSlider, newFile)
    if nargin ~= 3; return; end;


psv_message(data, 'Open and read file with phase space data',0); 

% choose, check and open file, save new default path
[data, fid, filename, filetype] = psv_open_file( data, usingSlider, newFile);
if (~ fid > 0)
    psv_message(data, 'Could not open the file!',1);
    return
end

% clear data.ps(data.my_axes) if it exists;
s_len = length(data.ps);
if(s_len >= data.my_axes)
    f_names = fieldnames(data.ps(data.my_axes));
    for i =1:length(f_names)
        % do not clear beamline info data
        if ~strcmp(char(f_names(i)),'bl') && ...
            isempty(strfind(char(f_names(i)),'r_axis')) && ...
            isempty(strfind(char(f_names(i)),'mesh_data'))
            data.ps(data.my_axes) = setfield(data.ps(data.my_axes),char(f_names(i)),[]);
        end
    end
end
%
% put filename in structure
data.ps(data.my_axes).filename = filename;
% put filetype in structure
data.ps(data.my_axes).filetype = filetype;
% setup units
data.ps(data.my_axes).unit   = {'m';'rad';'m';'rad';'m';'rel.';'A';'mm-mrad';'mm-mrad';'rel.';'V';'V'};
data.ps(data.my_axes).unit_e = {'m-rad';'m';'m-rad';'m';'m'};

% if filetype is of fooNNN.bar nature, look for other files (if 
% new file) and get all the files data
similarFileName = [];
if ( newFile )
    [pathstr, name, ext] = fileparts(filename);
    reg = [ '[0-9]+\' ext '$' ];
    name = [name  ext];
    similarFileName = regexp(name , reg, 'once' );
    leading = name(1:similarFileName-1);
end

list_dir = [];
if ~isempty(similarFileName)
    dirname = [pathstr filesep leading '*' ext ];
    list_dir = dir(dirname);
    list_dir = list_dir (find(~cellfun(@isempty,{list_dir(:).date}))); 
    z = zeros (1,length(list_dir));
    for i=1 : length(list_dir)
        u = regexp(list_dir(i).name, reg, 'once');
        if ~ isempty(u)
            z(i) = i;
        end
    end
    list_dir = list_dir(find (z>0));
end

if (~ isempty(list_dir) && length(list_dir) > 1)
    [ data, ierr ] = psv_get_beamline( data, list_dir );
end

%
% read data ---------------------------------------------------------------
psv_message(data, 'Read phase space data from file',1);
data = psv_read_data(data, fid,data.ps(data.my_axes).filename);
if(data.ps(data.my_axes).ierr == 1)
    return
end

psv_message(data, 'Analyze phase space data',1);
% phase space statistical moments -----------------------------------------
data = psv_ps_statistics( data );
%
psv_message(data, 'Plot phase space data',1);
% start by plotting longitudinal phasespace
data = psv_plot_ps(data, 0);

[pathstr, name, ext] = fileparts(data.ps(data.my_axes).filename);
message = [ 'Plot phase space data (' name ext ') - done' ];
psv_message(data, message,1);
