%-------------------choose, check and get file name ----------------------------
function [data, fid, filename, filetype] = psv_open_file(data, usingSlider, newFile) 
    filetype = '';
    filename = '';
    fid      = 0;

    % there are 3 ways to get the filename:
    % open select file box if newFile is true
    % get filename from array if usingSlider is true
    % or get filename from data.ps(data.my_axes).filename
    if newFile % open file selector
        % read default path structure or create empty default path
        default_path = ''; defaultpath1 = '';  defaultpath2 = '';
        if(exist('ps_viewer_data.mat','file') == 2)
            load('ps_viewer_data.mat');
            data.psv_defaults(1).path = defaultpath1;
            data.psv_defaults(2).path = defaultpath2;
            default_path = data.psv_defaults(data.my_axes).path;
        end
        [ filename_filetype pathname FilterIndex] = uigetfile( ...
            {'*.*', 'Astra input';'*.fmt1;*.fmt2;*.fmt3','CSRTrack input';'*.*','Elegant input'}, ...
            'Choose a Phasespace Data File', default_path);
        if(filename_filetype==0)
            % user cancelled
            fid=0;
            return
        end
        data.psv_defaults(data.my_axes).path = pathname;
        filename = [pathname filename_filetype];
        % delete old data
        data.ps(data.my_axes).mesh_data = [ ];
        data.ps(data.my_axes).bl = struct();
        if data.ps(data.my_axes).hor_axis > 5 % reset popup menu if we are in mesh mode
            data.ps(data.my_axes).hor_axis = 5;
        end
    elseif usingSlider % load filename from filename array
        ps_files      = data.ps(data.my_axes).bl.file_names;
        ps_file_index = data.ps(data.my_axes).bl.act_file;
        filename      = char(ps_files(ps_file_index));
        
    else % use stored filename
        filename = data.ps(data.my_axes).filename;
    end

    if(exist(filename,'file') ~= 2)
        display([ 'ERROR: file not ' filename]);
        return;
    end

    % open file
    fid=fopen(filename,'rt');
    if(fid==-1)
        uiwait(msgbox('Cannot open file (this should not happen)','IO Error','error'))
        fid=0;
        return
    end

    % Now look at and into the file and determine its type
    [dummy dummy file_id] = fileparts(filename);
    if  (strcmp(file_id,'.fmt3'))
        filetype='CSR_fmt3';
    elseif(strcmp(file_id,'.fmt2'))
        filetype='CSR_fmt2';
    elseif(strcmp(file_id,'.fmt1'))
        filetype='CSR_fmt1';
    else
        % avoid menu if a slider is calling
        if ~ newFile
            filetype = data.ps(data.my_axes).filetype;
        else
            if FilterIndex==1 
                filetype = 'ASTRA';
            elseif FilterIndex==3
                filetype = 'elegant';
            else
                fid = 0;
                return;
            end
        end
    end
    
    % save default
    defaultpath1 = data.psv_defaults(1).path;
    defaultpath2 = data.psv_defaults(2).path;
    save('ps_viewer_data.mat', 'defaultpath1', 'defaultpath2');
end