function [ data ] = psv_plot_ming_chi(data, hObject, eventdata, handles) %#ok<INUSD>
    if nargin == 0; return; end;    
    c    = 2.99792458E8;	    % m/s
    
    E0 =       data.psv_options(data.my_axes).plot.MeshPlot.MingChi.E0;
    K =        data.psv_options(data.my_axes).plot.MeshPlot.MingChi.K;
    beta =     data.psv_options(data.my_axes).plot.MeshPlot.MingChi.beta;
    lamu =     data.psv_options(data.my_axes).plot.MeshPlot.MingChi.lamu;
    %
    bin = data.ps(data.my_axes).bin;
    
    grid = data.ps(data.my_axes).b_stat.grid;
    I = data.ps(data.my_axes).Bcurrent(5).shape;
    
    slices = length(grid)-1;

    for ig=1:slices
        if(length(find(bin==ig)) > 10)
            slice_pos(ig) = 0.5*(grid(ig)+grid(ig+1)); 
            % mean cutrrent)
            I_mean(ig) = mean(I(find((I(:,1)> grid(ig)) & (I(:,1) < grid(ig+1))),2))

            %sigZ    = sl_rms(ig,5);
            sigZ(ig)   = data.ps(data.my_axes).slices(ig).b_stat.b_rms(5);
            %Ipk  = Q*c/sqrt(12)/sigZ;	% Amps
            q(ig) = I_mean(ig)/c*sqrt(12)*sigZ(ig);
            %emitN   = sl_emit(ig,1);
            emitN(ig)   = data.ps(data.my_axes).slices(ig).b_stat.b_emit(1);
            %esprd   = sl_rms(ig,6) * (gammag*0.511/1000.)/E0;
            esprd(ig)   = data.ps(data.my_axes).slices(ig).b_stat.b_rms(6) * (data.ps(data.my_axes).gamma*0.511/1000.)/E0;
            %
            beta(ig) = data.ps(data.my_axes).slices(ig).b_stat.b_twiss(2,1);
            %
            [Lg(ig),L1d,Psat(ig),rho,Lr,Ipk(ig),sigx,lamr(ig),Lg0,Lgssy(ig),Xi_sat(ig),Br(ig),Psatssy(ig)] = sase_fel_ssy(E0,K,q(ig),sigZ(ig),emitN(ig),beta(ig),esprd(ig),lamu);
            %
            if(ig == round(slices/2))
                disp('Input:')
                disp([' Beam Energy [GeV]: ' num2str(E0) ' av. beta function [m]: ', num2str(beta(ig))])
                disp([' Undulator Period [m]: ' num2str(lamu) ' K Parameter: ', num2str(K)])
                disp([' emittance [m]: ' num2str(emitN(ig)) ' Energy spread: ', num2str(esprd(ig))])
                disp([' slice charge: ' num2str(q(ig)) ' slice rms length: ', num2str(sigZ(ig))])
                disp(' ')
                disp('Output:')
                disp([' radiation wavelength [nm]: ' num2str(lamr(ig)*1e9) ' I_peak: ', num2str(Ipk(ig))])
                disp([' gain length [m]: ' num2str(Lg(ig)) ' FEL parameter rho: ', num2str(rho)])
            end
        else
            Lg(ig) = 0;
            Lgssy=0;
            Psat(ig) = 0;
        end
    end
    %
    %	SASE FEL Ming Xie formual to approximate the 3D gain length of a SASE FEL.
    %
    %    INPUTS:	E0:	    The electron beam energy [GeV]
    %		        K:	    The undulator parameter [ ]
    %		        Q:	    The bunch charge [C]
    %		        sigZ:	The rms e- bunch length (uniform dist. assumed) [m]
    %		        emitN:	The rms normalized transverse emittance (ex=ey) [m]
    %		        beta:	The mean beta function in the undulator (assumed ~constant) [m]
    %		        esprd:	The rms relative energy spread [unitless, e.g. 1E-4]
    %		        lamu:	The undulator perdiod [m]
    %
    %    OUTPUTS:	Lg:	    The 3D gain length [m]
    %		        L1d:	The 1D gain length (Lg >= L1d) [m]
    %		        Psat:	Estimated saturation power [TW]
    %		        rho:	FEL parameter [ ]
    %		        Lr:	    Rayleigh range [m]
    %		        Ipk:	Peak current of e- beam for uniform z-dist. [A]
    %		        sigx:	rms transverse beam size (x=sy, assumed ~constant) [m]
    %		        lamr:	Radiation wavelength [m]
    %
    mingdata = struct();
    mingdata.x1=slice_pos;
    mingdata.y1=Lg;
    mingdata.x2=slice_pos;
    mingdata.y2=Psat*1000;
    mingdata.x3=slice_pos;
    mingdata.y3=Ipk;
    
    ssyfig = figure('Name','SSY Plot', 'Color', get(0,'defaultUicontrolBackgroundColor'));
    subplot(1,4,1)
    plot(slice_pos,Lgssy)
    ylabel('SSY Field Gain Length [m]');
    xlabel('s along the Bunch [m]')

    subplot(1,4,2)
    plot(slice_pos,Br)
    ylabel('SSY Brightness')
    xlabel('s along the Bunch [m]')

    subplot(1,4,3)
    plot(slice_pos,Xi_sat)
    ylabel('SSY Degree of Coherence')
    xlabel('s along the Bunch [m]')
    v = axis;
    axis( [ v(1) v(2) 0 1 ] );

    subplot(1,4,4)
    plot(slice_pos,Psatssy)
    ylabel('SSY Power [GW]')
    xlabel('s along the Bunch [m]')
    uicontrol(ssyfig, 'Style', 'pushbutton', 'String', 'MingChi', 'Position', [1 1 80 20], 'Callback', @open_ming_chi, 'UserData', mingdata);
end

function open_ming_chi(hObject, eventdata) %#ok<INUSD>
    mingdata = get( hObject, 'UserData' );
    mingfig = figure('Name', 'Ming Chi Plot','Color', get(0,'defaultUicontrolBackgroundColor'));
    subplot(1,3,1)
    plot(mingdata.x1,mingdata.y1)
    ylabel('Ming Chi Power Gain Length [m]')
    xlabel('s along the Bunch [m]')

    subplot(1,3,2)
    plot(mingdata.x2,mingdata.y2)
    ylabel('Ming Chi Saturation Power [GW]')
    xlabel('s along the Bunch [m]')
    %
    subplot(1,3,3)
    plot(mingdata.x3,mingdata.y3)
    ylabel('I [A]')
    xlabel('s along the Bunch [m]')
    uicontrol(mingfig, 'Style', 'pushbutton', 'String', 'Close', 'Position', [1 1 60 20], 'Callback', @close_ming_chi, 'UserData', mingfig);
end

function close_ming_chi(hObject, eventdata) %#ok<INUSD>
    mingdata = get( hObject, 'UserData');
    close(mingdata);
end
