function [ data ] = psv_print_axes(data)  %#ok<INUSD>
% puts ps_viewer axes into seperate window for editing and printing
    if nargin == 0 || isempty( data ); return; end;
    
    zoomfig  = figure('Tag', 'psvzoomfig');
    axes('Tag', 'psvzoomaxes', 'Parent', zoomfig, 'Units', 'Pixels');
    % horizontal and vertical axis
    hor_axis = data.ps(data.my_axes).hor_axis;
    ver_axis = data.ps(data.my_axes).ver_axis;
    mesh     = get(data.psv_hnames.mesh_handles(data.my_axes), 'Value');  % mesh plot
    fix      = get(data.psv_hnames.square_handles(data.my_axes), 'Value'); % use fixed aspect ratio
    slice    = -1; % disable slice by default
    if(get(data.psv_hnames.slice_handles(6,data.my_axes),'Value'))
        slice = get(data.psv_hnames.slice_handles(1,data.my_axes),'Value');
        if (slice == 0) % not yet set, (toggle button pressed)
            slice = round(( data.ps(data.my_axes).nbins + 1)/2);
        end
    end
    % plot phasespace    
    tmp_data = psv_plot_phasespace( data, hor_axis, ver_axis, mesh, fix, slice); 

    % units for the axes
    unit = data.ps(data.my_axes).unit;
    set(gca, 'Units', 'normalized'); % enable resize again
    if isfield(tmp_data.ps(data.my_axes), 'cbar_axes') && ~isempty(tmp_data.ps(data.my_axes).cbar_axes)
        set(tmp_data.ps(data.my_axes).cbar_axes, 'Units', 'normalized'); % enable resize of new colorbar
    end

    % rms values
    b_rms = data.ps(data.my_axes).b_stat.b_rms;
    
    xlabels = {'horizontal position';'horizontal angle';'vertical position';'vertical angle';...
        'longitudinal position';'energy deviation';'distance along the beam line'};
    ylabels = {'horizontal position';'horizontal angle';'vertical position';'vertical angle';...
        'longitudinal position';'energy deviation';...
        'Current';'Horizontal Slice Emittance';'Vertical Slice Emittance';'Slice Energy Spread';...
        'Longitudinal CSR Field';'Transverse CSR Field'};
    ylabels7= {'horizontal beam size (RMS)';'horizontal angle dist. (RMS)';'vertical beam size (RMS)';...
        'vertical angle dist. (RMS)';'longitudinal beam size (RMS)';'energy spread (RMS)';...
        'Horizontal Emittance'      ;'Vertical Emittance';          'Longitudinal Emittance';...
        'Horizontal Beta-Function';  'Vertical Beta-Function';      'Longitudinal Beta-Function'};
    unit7   = {'m';'rad';'m';'rad';'m';'rel.';'m-rad';'m-rad';'m';'m';'m';'m'};
    title('edit title');
    if(hor_axis~=7)
        xlabel([char(xlabels(hor_axis)) ' [' char(unit(hor_axis)) ']' '  (RMS: ' char(num2str(b_rms(hor_axis),'%0.2e')) ' [' char(unit(hor_axis)) '])']);
    else
        xlabel([char(xlabels(hor_axis)) ' [m]']);
    end
    if(hor_axis~=7)
        if(ver_axis > 6)
            ylabel([char(ylabels(ver_axis)) ' [' char(unit(ver_axis)) ']']);
        else
            ylabel([char(ylabels(ver_axis)) ' [' char(unit(ver_axis)) ']' '  (RMS: ' char(num2str(b_rms(ver_axis),'%0.2e')) ' [' char(unit(ver_axis)) '])']);
        end
    else
        ylabel([char(ylabels7(ver_axis)) ' [' char(unit7(ver_axis)) ']']);
    end
end
