%--------------------------------------------------------------------------
%---------------------  generate particle distributions -------------------
function [ data ] = psv_ps_generator( data )
%
if nargin ~= 1; return ; end;
psv_message(data, 'Generating Particle Distribution',0);
% structure for phase space
%
% ask for orders
%
prompt = {
    'Number of particles: ';...
    'Bunch Charge [nC] :';...
    'Mean energy [MeV]: ';...
    'Horizontal emittance (normalized) [mm-mrad]: ';...
    'Vertical emittance (normalized)[mm-mrad]: ';...
    'Bunch Length [mm]: ';...
    'Relative Energy Spread: ';...
    'Distribution Type. G is Gaussian, E is evenly distributed :';
    };
defaultanswer = {
    num2str(10000);...
    num2str(1);...
    num2str(150);...
    num2str(1e-6);...
    num2str(1e-6);...
    num2str(1e-3);...
    num2str(1e-3);...
    'G'
    };
name = 'Particle Generator Parameters';
numlines = 1;
answer = inputdlg(prompt,name,numlines,defaultanswer);
if(isempty(answer))
    return
end
%
NumPart     = str2num(char(answer(1)));
q_total     = str2num(char(answer(2)))*1e-9;
gammag      = str2num(char(answer(3)))/0.511;
e_x         = str2num(char(answer(4)));
e_y         = str2num(char(answer(5)));
s_z         = str2num(char(answer(6)))*1e-3;
d_e         = str2num(char(answer(7)));
dist_type   =         char(answer(8)) ; 
%
if(strcmp(dist_type,'G'))
    phasespace  = randn(NumPart,6);
    % weight contains beam sizes for alpha=0 and beta =1
    weight      =  [sqrt(e_x/gammag), sqrt(e_x/gammag),...
                    sqrt(e_y/gammag), sqrt(e_y/gammag),... 
                    s_z             , d_e/(gammag*0.511)];
else
    if strcmp(dist_type, 'E')
        phasespace  = rand(NumPart,6);
        % weight contains beam sizes for alpha=0 and beta =1
        weight      =  [sqrt(e_x/gammag), sqrt(e_x/gammag),...
                        sqrt(e_y/gammag), sqrt(e_y/gammag),... 
                        s_z             , d_e/(gammag*0.511)];
    else
        return
    end
end                    

for i=1:6
    phasespace(:,i) = phasespace(:,i) * weight(i);
end
phasespace(:,7) =  q_total/NumPart;

%
% start position -0.1 m before lattice begin to avoid field boundaries
ct = -0.1;
data.ps(data.my_axes).ct = ct;
reference = [0 0 0 0 ct 0];          
data.ps(data.my_axes).reference =  reference;
data.ps(data.my_axes).phasespace = phasespace;
data.ps(data.my_axes).q_total =    q_total;
data.ps(data.my_axes).gamma =      gammag;
data.ps(data.my_axes).bl = [ ];
data.ps(data.my_axes).mesh_data = [ ];
data.ps(data.my_axes).fields = [ ];
% setup units
data.ps(data.my_axes).unit   = {'m';'rad';'m';'rad';'m';'rel.';'A';'mm-mrad';'mm-mrad';'rel.';'V';'V'};
data.ps(data.my_axes).unit_e = {'m-rad';'m';'m-rad';'m';'m'};
if data.ps(data.my_axes).hor_axis > 6 % reset popup menu if we are in mesh mode
    data.ps(data.my_axes).hor_axis = 6;
end
if data.ps(data.my_axes).ver_axis > 10 % reset popup menu if we are in field mode
    data.ps(data.my_axes).ver_axis = 10;
end

psv_message(data, 'Generating Particle Distribution done',1);
%
psv_message(data, 'Analyze phase space data',1);
% phase space statistical moments -----------------------------------------
data = psv_ps_statistics( data );
%
psv_message(data, 'Plot phase space data',1);
% start by plotting longitudinal phasespace
data = psv_plot_ps(data, 0);

psv_message(data, 'Plot phase space data - done',1);
% 