%--------------------------------   read phase space data -----------------

function [ data ] = psv_read_data(data, fid,filename)
%
if nargin ~= 3
    return;
end
ierr = 0;
filetype = data.ps(data.my_axes).filetype;
%
if(strcmp(filetype,'CSR_fmt1') | strcmp(filetype,'CSR_fmt2'))
    %   CSRtrack coordinates:   ds  dh  dv  dps dph dpv  in cartesien
    %                                                    coordinates, x
    %                                                    directed in flight
    %                                                    direction
    %   ps_viewer coordinates:  x   x'  y   y'  s   delta
    psv_index = [2 5 3 6 1 4 7];
    % get first line
    line = fgetl(fid);
    % read first number (position)
    [ct] = sscanf(line,'%e',[1 1]);
    % 2nd line is reference particle
    reference = fscanf(fid,'%e %e %e %e %e %e %e \n',[1 7]);
    reference = reference(psv_index);
    % take gammag from momentum of reference particle and zero reference
    % particle
    e0 = reference(6);
    gammag=e0/(0.511*1e6);                                                   
    %                                                      
    % read phasespace
    [ data phasespace ] = psv_read_data_extern(data,fid,filename,7);
    
    % add zeros for reference particle    
    phasespace = [0 0 0 0 0 0 reference(7);phasespace];
    
    % shuffle phasespace

    %phasespace = [  phasespace(:,2)    phasespace(:,5)/e0 phasespace(:,3)    ...
    %                phasespace(:,6)/e0 phasespace(:,1)    phasespace(:,4)/e0 ...
    %                phasespace(:,7)];
    phasespace = phasespace(:,psv_index);
    for i = 2:2:6
        phasespace(:,i) = phasespace(:,i)/e0;
    end
    % store total charge
    q_total = sum(phasespace(:,7));
    fields=[];
    %
elseif(strcmp(filetype,'CSR_fmt3'))
    %
    % get first line
    line = fgetl(fid);
    % check for number of columns
    column_number = 0;
    found = 0;
    for i=1:length(line)
        if(strcmp(line(i),' '))
            if(~found) 
                column_number = column_number + 1;
                found = 1;
            end
        else
            found = 0;
        end
    end
    switch column_number
        case 12
            % read position and energy from first line
            dummy = sscanf(line,'%e %e',[2 1]);
            ct =dummy(1);
            gammag = dummy(2);
            % 2nd line is reference particle
            reference = fscanf(fid,'%e %e %e %e %e %e %e \n',[12 1]);
            reference = reference(1:7);
            reference(6) = (reference(6)+1)*gammag*0.511e6;
            
            % phasespace and fields
            [ data mat_in ] = psv_read_data_extern(data,fid,filename,12);
   
            phasespace  = mat_in(:,1: 7);
            fields      = mat_in(:,8:10);
            %   store total charge
            q_total     =   sum(phasespace(:,7));
            %
            %    [ps_r index] = read_fmt3_mex(filename_out,0);
            %     ps_r=ps_r';
            %     ct          =   ps_r(1,1);
            %     gammag      =   ps_r(1,2);
            %     reference   =   ps_r(2,1:12);
            %     phasespace  =   ps_r(3:length(ps_r),1:7);
            %     fields      =   ps_r(3:length(ps_r),8:10);
            %     q_total     =   sum(phasespace(:,7));
            %
        case 13
            default = 'Cancel';
            ButtonName = questdlg(['This is a 13 column fmt3 file, probably a single particle monitor.' ...
                                   'Proceed with special beam line plot?'],'fmt3 Warning','Cancel','Yes',default);
            switch ButtonName
                case 'Cancel'
                    data.ps(data.my_axes).ierr = 1;
                    return
                case 'Yes'
                    uiwait(msgbox('Not yet implemented...','Hoho!'));
                    data.ps(data.my_axes).ierr = 1;
                    return
            end % switch
        otherwise
            uiwait(msgbox('Format fmt3 must have 12 (or 13) columns!','IO Error','error'))
            data.ps(data.my_axes).ierr = 1;
            return
    end
elseif(strcmp(filetype,'ASTRA'))
    try
        % 1nd line is reference particle
        reference = fscanf(fid,'%e %e %e %e %e %e %e %e %e %e \n',[10 1]); 
        ct = reference(7)*0.299792458; %c in m/ns
        % shuffle reference
        reference=[ reference(1) reference(4) reference(2) ...
                        reference(5) reference(3) reference(6) ...
                        reference(8) ];
        % take gammag from momentum of reference particle
        e0 = reference(6);
        gammag=e0/(0.511*1e6);
        % phasespace 
        [data phasespace ] = psv_read_data_extern(data,fid,filename,10);

        phase_remove = find(phasespace(:,10) < 0);
        phasespace(phase_remove,:) = [];
        clear phase_remove;

        % shuffle phasespace 
        phasespace = [  phasespace(:,1)-reference(1)    (phasespace(:,4)-reference(4))/e0 phasespace(:,2)-reference(2)    ...
                        (phasespace(:,5)-reference(5))/e0 phasespace(:,3)    phasespace(:,6)/e0 ...
                        abs(phasespace(:,8)*10^-9)];
        % normalize charge distribution to 1, store total charge
        q_total = sum(phasespace(:,7));
        fields=[];
    catch ME
        %
        uiwait(msgbox('ASTRA import error','PSC_READ_DATA','error'))
        ierr = 1;
    end
%
elseif(strcmp(filetype,'elegant'))
    % this assumes ascii elegant output
    gammag=0;
    q_total = 0;
    
    % read header
    coord_names={'x';'xp';'y';'yp';'t';'p'};
    coord_index=[];
    line = fgetl(fid);
    if(~strcmp(line(1:5),'SDDS1'))
        uiwait(msgbox('elegant format: First line should be ''SDDS1'' ','PSC_READ_DATA','error'))
    end
    do_read_header = 1;
    header_index = 0;
    parameter_index = 0;
    while(do_read_header)
        header_index = header_index + 1;
        line = fgetl(fid);
        % disp(line)
        is_and = strcmp(line(1),'&');
        if(is_and)
            if      (strcmp(line(1:12),'&description'))
            elseif  (strcmp(line(1:10),'&parameter'))
                parameter_index = parameter_index + 1;
                if      (~isempty(findstr(line,'Step')))
                            parameter_name(parameter_index) = {'Step'};
                elseif  (~isempty(findstr(line,'Charge')))
                            parameter_name(parameter_index) = {'Charge'};
                elseif  (~isempty(findstr(line,'Particles')))
                            parameter_name(parameter_index) = {'Particles'};
                elseif  (~isempty(findstr(line,'pCentral')))
                            parameter_name(parameter_index) = {'pCentral'};
                else
                        parameter_name(parameter_index) = {''};
                end
            elseif  (strcmp(line(1:7),'&column'))
                    % which column?
                    line_ind = strfind(line,'name=') + length('name=');
                    if      (strcmp(line(line_ind+1),','))
                        coord_name = line(line_ind);
                    elseif  (strcmp(line(line_ind+2),','))
                        coord_name = line(line_ind:line_ind+1);
                    else
                        coord_name=[];
                    end
                    % find coordinate index
                    if(~isempty(coord_name))
                        coord_index = ...
                         [coord_index strmatch(coord_name,coord_names,'exact')];
                    else
                        coord_index = ...
                         [coord_index 0];
                    end
             end
        elseif(strcmp(line(1),'!'))
            break
        end
    end
    % read parameter data
    if(parameter_index > 0)
        for i = 1:length(parameter_name)
            p_name = char(parameter_name(i));
            line = fgetl(fid);
            switch p_name
                case 'Step'
                case 'Charge'
                    q_total = str2num(line);
                case 'Particles'
                    n_part  = str2num(line);
                case 'pCentral'
                    gammag = str2num(line);
            end
        end
    end
    % read (maybe again) number of particles
    p_number = str2num(fgetl(fid));
    if(exist('n_part'))
        if(p_number ~= n_part)
            disp()
            uiwait(msgbox('elegant format: particle numbers do not match!','PSC_READ_DATA','error'))
            ierr=1
            return
        end
    end
    %
    if(length(coord_index) ~=7)
        disp('Warning in psv_read_data (elegant format): There are more than 7 columns for the phase space description!')
    end
    outdummy  = zeros(p_number,length(coord_index));
    display('for now loading elegant files doesnt support fast loading');
    outdummy  = fscanf(fid,'%e %e %e %e %e %e %e %e',[length(coord_index) p_number])';
    %
    phasespace  = zeros(p_number,7);
    % if the central energy is not passed via a parameter
    if(gammag == 0)
        gammag = mean(outdummy(:,find(coord_index==6)));
    end
    %
    for i = 1:p_number
        gammag_p = outdummy(i,find(coord_index==6));
        delta_e = (gammag_p-gammag)/gammag;
        
        phasespace(i,:)  =  [outdummy(i,find(coord_index==1)) outdummy(i,find(coord_index==2)) ...
             outdummy(i,find(coord_index==3)) outdummy(i,find(coord_index==4)) ...
             -outdummy(i,find(coord_index==5))*2.998e8 delta_e ...
             1/p_number];
    end
    % subtract average longitudinal position and put it into reference particle
    s_pos = mean(phasespace(:,5));
    phasespace(:,5) = phasespace(:,5) - s_pos;
    % stuff we are not getting from elegant
    ct          =   -s_pos;
    reference   =   zeros(1,7);
    % energy
    if(gammag ~= 0)
        reference(6) = gammag*0.511*1e6;
    else
        beam_energy = inputdlg('Please specify the beam energy (MeV)');
        e0 = str2num(beam_energy)*1e6;
        reference(6) = e0;
    end
    %    
    fields = [];
else
    uiwait(msgbox('No file tpye information available','PSC_READ_DATA','error'))
    ierr = 1;
end
fclose(fid);
%
%
% fill phase space structure-----------------------------------------------
data.ps(data.my_axes).ierr = ierr;
if(ierr == 0)
    if(q_total == 0)
        q_total = 1e-9;
        uiwait(msgbox('No information on bunch charge from elegant, assume 1 nC!','PSC_READ_DATA','warn'))
    end
    data.ps(data.my_axes).reference =  reference;
    data.ps(data.my_axes).phasespace = phasespace;
    data.ps(data.my_axes).fields =     fields;
    data.ps(data.my_axes).q_total =    q_total;
    data.ps(data.my_axes).gamma =      gammag;
    data.ps(data.my_axes).ct =         ct;
    data.ps(data.my_axes).ft =         filetype;
end
end

function [ data phasespace ] = psv_read_data_extern(data,fid,filename,col)

    offset = ftell(fid);
    try
        m = memmapfile(filename,'Offset',offset);
        temp = m.data;
    	phasespace = ascii2array(temp);
    catch ME
        % error case: use old style
        idSegLast = regexp(ME.identifier, '(?<=:)\w+$', 'match');
        fseek(fid, offset, 'bof');
        switch(col)
            case 7
                phasespace = fscanf(fid,'%e %e %e %e %e %e %e',[7 inf])';
            case 10
                phasespace = fscanf(fid,'%e %e %e %e %e %e %e %e %e %e',[10 inf])';
            case 12
                phasespace = fscanf(fid,'%e %e %e %e %e %e %e %e %e %e %e %e',[12 inf])';
            case 13
                phasespace = fscanf(fid,'%e %e %e %e %e %e %e %e %e %e %e %e %e',[13 inf])';
            otherwise
        end
        if strcmp(idSegLast,'UndefinedFunction') == 1
            psv_message(data, 'WARNING: external c function "ascii2array" not found, use slow file input function',1);
            psv_message(data, ' ',0);
        else
            info = char(strcat('WARNING: exception "ascii2array" : ', idSegLast));
            psv_message(data, info ,1);
            psv_message(data, ' ',0);
        end
    end
  	clear temp;
    clear m;
end