function [ data ] = psv_set_options(data, dialogName)
if nargin == 0; return; end;
if ~isempty(strfind(dialogName,'Plot_Sum'))
    prompt = {
        'Center plot around peak current? (0/1)';
        'Width of window (in FWHM''s)';
        'Upper Limit for Slice Emittance';...
        'Upper Limit for Beta-Mismatch'...
        };
    defaultanswer = {
        num2str(data.psv_options(data.my_axes).plot.SumPlot.center_peak);... = 0;
        num2str(data.psv_options(data.my_axes).plot.SumPlot.s_window_width);... = 3;
        num2str(data.psv_options(data.my_axes).plot.SumPlot.sl_e_lim);...
        num2str(data.psv_options(data.my_axes).plot.SumPlot.betab_lim)...
        };
    name = ['SumPlot Options [' num2str(data.my_axes) ']'];
    numlines = 1;
    answer = inputdlg(prompt,name,numlines,defaultanswer);
    if(isempty(answer))
        return
    end
    %
    % make structure for plotting options
    data.psv_options(data.my_axes).plot.SumPlot.center_peak    = str2double(char(answer(1)));
    data.psv_options(data.my_axes).plot.SumPlot.s_window_width = str2double(char(answer(2)));
    data.psv_options(data.my_axes).plot.SumPlot.sl_e_lim       = str2double(char(answer(3)));
    data.psv_options(data.my_axes).plot.SumPlot.betab_lim      = str2double(char(answer(4)));

elseif ~isempty(strfind(dialogName,'Plot_Plot'))
    prompt = {

        'Particle bins equidistant? (0/1)';
        'Number of bins';
        'Particle bins equally populated? (0/1)';
        'Number of particles per bin';
        'Manually override step size for equal number binning? (0/1)';
        'Step size for equal number binning (sigma/10 in auto mode)';
        'Manually override width of gauss for current curve smoothing? (0/1)';
        'Width of gauss for current curve smoothing (s-rms in auto mode)';
        'Fraction of Particles to be plotted'...
        };
    defaultanswer = {
        num2str(data.psv_options(data.my_axes).plot.bin.equidistant);... = 0;
        num2str(data.psv_options(data.my_axes).plot.bin.nbins);... = 51;
        num2str(data.psv_options(data.my_axes).plot.bin.equalnum);... = 1;
        num2str(data.psv_options(data.my_axes).plot.bin.nparts);... = 10000;
        num2str(data.psv_options(data.my_axes).plot.bin.manstep);... = 0;
        num2str(data.psv_options(data.my_axes).plot.bin.stepsize);... = 1e-5;
        num2str(data.psv_options(data.my_axes).plot.bin.man_g_sigma);... = 0;
        num2str(data.psv_options(data.my_axes).plot.bin.g_sigma);... = 0;
        num2str(data.psv_options(data.my_axes).plot.p_frac);... = 1;
        };
    name = ['General Plotting Options [' num2str(data.my_axes) ']'];
    numlines = 1;
    answer = inputdlg(prompt,name,numlines,defaultanswer);
    if(isempty(answer))
        return
    end
    %
    % make structure for plotting options
    data.psv_options(data.my_axes).plot.bin.equidistant        = str2double(char(answer(1)));
    data.psv_options(data.my_axes).plot.bin.nbins              = str2double(char(answer(2)));
    data.psv_options(data.my_axes).plot.bin.equalnum           = str2double(char(answer(3)));
    data.psv_options(data.my_axes).plot.bin.nparts             = str2double(char(answer(4)));
    data.psv_options(data.my_axes).plot.bin.manstep            = str2double(char(answer(5)));
    data.psv_options(data.my_axes).plot.bin.stepsize           = str2double(char(answer(6)));
    data.psv_options(data.my_axes).plot.bin.man_g_sigma        = str2double(char(answer(7)));
    data.psv_options(data.my_axes).plot.bin.g_sigma            = str2double(char(answer(8)));
    data.psv_options(data.my_axes).plot.p_frac                 = str2double(char(answer(9)));

    if data.psv_options(data.my_axes).plot.bin.man_g_sigma && data.psv_options(data.my_axes).plot.bin.g_sigma <= 0
        psv_message( data, 'Sigma of Gauss for curve smoothing must be > 0! ',0);
        psv_message( data, 'Manually override width of gauss for current curve smoothing DISABLED',0); 
        data.psv_options(data.my_axes).plot.bin.man_g_sigma = false;
        beep;
    end

    % if phase space data exists, re-calculate phase space parameters
    if isfield(data.ps(data.my_axes),'phasespace') && ~ isempty(data.ps(data.my_axes).phasespace)
        psv_message(data, 'Analyze phase space data',0);
        % phase space statistical moments
        data = psv_ps_statistics( data );
        % re-plot phase space
        psv_message(data, 'Plot phase space data',0);
        % start by plotting longitudinal phasespace
        data = psv_plot_ps(data, 0);
    end
    
elseif ~isempty(strfind(dialogName,'Plot_Mesh'))
    prompt = {
        'Undulator Beam energy';...
        'Undulator K parameter';...
        'Undulator Beta Function (av.)';...
        'Undulator Period Length'...
        };
    defaultanswer = {
        num2str(data.psv_options(data.my_axes).plot.MeshPlot.MingChi.E0);...
        num2str(data.psv_options(data.my_axes).plot.MeshPlot.MingChi.K);...
        num2str(data.psv_options(data.my_axes).plot.MeshPlot.MingChi.beta);...
        num2str(data.psv_options(data.my_axes).plot.MeshPlot.MingChi.lamu)... 
        };
    name = 'Plotting Options';
    numlines = 1;
    answer = inputdlg(prompt,name,numlines,defaultanswer);
    if(isempty(answer))
        return
    end
    %
    % make structure for plotting options
    data.psv_options(data.my_axes).plot.MeshPlot.MingChi.yn   = 1;
    data.psv_options(data.my_axes).plot.MeshPlot.MingChi.E0   = str2double(char(answer(2)));
    data.psv_options(data.my_axes).plot.MeshPlot.MingChi.K    = str2double(char(answer(3)));
    data.psv_options(data.my_axes).plot.MeshPlot.MingChi.beta = str2double(char(answer(4)));
    data.psv_options(data.my_axes).plot.MeshPlot.MingChi.lamu = str2double(char(answer(5)));
end   