%---------------- save data in old CSRtrack format ------------------------
function data = psv_write_file( data, filename )
    if nargin ~= 2; return ; end;
%
fid = fopen(filename,'w');
%
% get phase space parameters
phasespace  = data.ps(data.my_axes).phasespace;
reference   = data.ps(data.my_axes).reference;
gammag      = data.ps(data.my_axes).gamma;
ct          = data.ps(data.my_axes).ct;
q_total     = data.ps(data.my_axes).q_total;
%
beam_E = gammag*0.511*1e6; % beam energy in eV
%
% which type of outputfile?
[dummy1 dummy2 type] = fileparts(filename);
%
switch type
    case '.fmt1'
        % write first line (position and zeros)
        fprintf(fid,'%e %e %e %e %e %e %e \n',[ct 0 0 0 0 0 0]);
        % shuffle phasespace
        %   CSRtrack coordinates:   ds  dh  dv  dps dph dpv  in cartesien
        %                                                    coordinates, x
        %                                                    directed in flight
        %                                                    direction
        %   ps_viewer coordinates:  x   x'  y   y'  s   delta
        csr_index = [5 1 3 6 2 4];
        % puts reference particle momentum in x direction 
        % (usual beam line direction) on energy, otherwise zeroes 
        reference = reference(csr_index);
        reference(4)=beam_E;
        % 
        % phasespace
        p=[phasespace(:,5) phasespace(:,1) phasespace(:,3) ...
            phasespace(:,6)*beam_E phasespace(:,2)*beam_E phasespace(:,4)*beam_E phasespace(:,7)];
        % put reference back
        p(1,:) = [reference 0];
        fprintf(fid,'%e %e %e %e %e %e %e \n',p');
        %
        %
    case '.ini'
         % write first line (position and zeros)
        fprintf(fid,'%e %e %e %e %e %e %e %e %d %d\n',[0 0 reference(5) 0 0 beam_E 0 0 1 5]);
%             
%     phasespace = [  phasespace(:,1)-reference(1)    (phasespace(:,4)-reference(4))/e0 phasespace(:,2)-reference(2)    ...
%                     (phasespace(:,5)-reference(5))/e0 phasespace(:,3)    phasespace(:,6)/e0 ...
%                     abs(phasespace(:,8)*10^-9)];
%     % normalize charge distribution to 1, store total charge
%     q_total = sum(phasespace(:,7));
        % shuffle phasespace 
        p = [phasespace(:,1) phasespace(:,3) phasespace(:,5) ...
                      phasespace(:,2)*beam_E phasespace(:,4)*beam_E phasespace(:,6)*beam_E ...
                      phasespace(:,5)/2.99792 phasespace(:,7) (zeros(1,length(phasespace))+1)' (zeros(1,length(phasespace))+5)'];
        fprintf(fid,'%e %e %e %e %e %e %e %e %d %d\n',p');
        %
        %
    case {'.ele','.sdds'}
        % write header
        fprintf(fid,...
        ['SDDS1 \n' ...
         '&parameter name=pCentral, symbol="p$bcen$n", units="m$be$nc", type=double, &end \n' ...    
        '&parameter name=Charge, units=C, description="Beam charge", type=double, &end \n' ...
        '&parameter name=Particles, type=long, &end \n' ...
        '&column name=x, units=m, type=double,  &end \n' ...
        '&column name=xp, symbol=x'', type=double,  &end \n' ...
        '&column name=y, units=m, type=double,  &end \n' ...
        '&column name=yp, symbol=y'', type=double,  &end \n' ...
        '&column name=t, units=s, type=double,  &end \n' ...
        '&column name=p, units="m$be$nc", type=double,  &end \n' ...    
        '&column name=particleID, type=long,  &end \n' ...
        '&data mode=ascii, &end \n' ...
        '! page number 1 \n %e \n %e \n %d \n                   %d \n'] ...
         ,gammag, q_total, length(phasespace), length(phasespace));
        %
        % write phasespace
        %   ps_viewer coordinates:  x   x'  y   y'  s   delta
        %     elegant coordinates:  x   x'  y   y'  t   E
        %
        id = zeros(length(phasespace),1);
        for i = 1: length(phasespace)
            id(i)=i;
        end
        p=[phasespace(:,1) phasespace(:,2) phasespace(:,3) ...
           phasespace(:,4) (ct - phasespace(:,5))/2.998e8 (gammag + phasespace(:,6)*gammag) id];
        % put reference back
%         reference = [0 0 0 0 ct/2.998e8 beam_E 1]; 
%         p(1,1:6) = reference;
        fprintf(fid,'%15.8e %15.8e %15.8e %15.8e %15.8e %15.8e %d \n',p');
end
fclose(fid);
